"""
テスト関連モデル
"""

from dataclasses import dataclass
from typing import List

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
# noinspection PyClassHasNoInit
class ConfigJsonModel:
    """
    設定JSONモデル
    """
    base_url: str
    username: str
    password: str


@dataclass_json
@dataclass
# noinspection PyClassHasNoInit
class BulletinBoardGetAllModel:
    """
    掲示板リスト取得テストモデル
    """
    item_id: str
    title: str
    target_s: str
    target_p: str
    flag_id: str
    unread_id: str
    is_attention: bool
    is_flag: bool
    is_unread: bool


@dataclass_json
@dataclass
# noinspection PyClassHasNoInit
class PublicationPeriodModel:
    """
    公開期間モデル
    """
    start_date: str
    end_date: str


@dataclass_json
@dataclass
# noinspection PyClassHasNoInit
class BulletinBoardGetDetailsModel:
    """
    掲示詳細取得テストモデル
    """
    item_id: str
    title: str
    author: str
    category: str
    content_html: str
    publication_period: PublicationPeriodModel


@dataclass_json
@dataclass
# noinspection PyClassHasNoInit
class TestsJsonModel:
    """
    テスト用JSONモデル
    """
    bulletinboard_get_all: List[BulletinBoardGetAllModel]
    bulletinboard_get_details: List[BulletinBoardGetDetailsModel]



"""
クラス
"""
import re
from typing import List

from unipa import Unipa
from unipa.Classes.UnipaClass import UnipaClass, UnipaClassLectureAt


class UnipaClasses:
    """
    クラス
    """

    def __init__(self,
                 unipa: Unipa):
        """
        コンストラクタ
        """
        self.unipa = unipa
        self.logger = unipa.logger

    def get_all(self) -> List[UnipaClass]:
        """
        履修中の全クラスを取得する
        """

        info_item = next(filter(lambda x: x.name == "クラスプロファイル", self.unipa.get_info_items()))
        soup = self.unipa.request_from_info(info_item)

        yobi_panels = soup.select(".yobiContArea")
        classes = []
        for yobi_panel in yobi_panels:
            day_of_week = yobi_panel.find("span", {"class": "ui-panel-title"}).text
            period_of_time_elements = yobi_panel.select("div.classJigen")
            class_lists = yobi_panel.select("div.classList")
            for period_of_time_element, class_list in zip(period_of_time_elements, class_lists):
                match = re.search(r"\d+", period_of_time_element.text)
                if match is None:
                    continue
                period_of_time = int(match.group()) if match is not None else None
                class_elements = class_list.select("div.ui-outputpanel")

                lecture_at = UnipaClassLectureAt(day_of_week, period_of_time)
                for class_element in class_elements:
                    a_tag = class_element.find("a")

                    name = a_tag.text
                    internal_id = a_tag.get("id")
                    class_raw = class_element.text
                    match = re.search("\\(([A-Za-z0-9]+)\\)", class_raw)
                    class_id = match.group(1) if match is not None else None

                    classes.append(UnipaClass(name, internal_id, class_id, lecture_at))

        return classes



# coding: utf-8
"""
Library for get various information about UNIVERSAL PASSPORT.

UNIVERSAL PASSPORT のさまざまな情報を取得するためのライブラリです。
"""
import logging
from typing import List, Optional
from urllib.parse import urljoin

import requests
from bs4 import BeautifulSoup
from requests import Response

from unipa.errors import UnipaInternalError, UnipaLoginError, UnipaNotLoggedIn
from unipa.unipa_utils import UnipaInfoItem, UnipaNavItem, UnipaRequestUrl, UnipaUtils


class UnipaToken:
    """
    トークンクラス
    """

    def __init__(self,
                 rx_token: str,
                 rx_login_key: str,
                 rx_device_kbn: str,
                 rx_login_type: str,
                 javax_view_state: str):
        """
        トークンクラス コンストラクタ

        Args:
            rx_token: トークン
            rx_login_key: ログインキー
            rx_device_kbn: デバイス区分
            rx_login_type: ログイン種別
            javax_view_state: View state
        """
        self._rx_token = rx_token
        self._rx_login_key = rx_login_key
        self._rx_device_kbn = rx_device_kbn
        self._rx_login_type = rx_login_type
        self._javax_view_state = javax_view_state

    @property
    def rx_token(self) -> str:
        """
        トークン

        Returns:
            str: トークン
        """
        return self._rx_token

    @property
    def rx_login_key(self) -> str:
        """
        ログインキー

        Returns:
            str: ログインキー
        """
        return self._rx_login_key

    @property
    def rx_device_kbn(self) -> str:
        """
        デバイス区分

        Returns:
            str: デバイス区分
        """
        return self._rx_device_kbn

    @property
    def rx_login_type(self) -> str:
        """
        ログイン種別

        Returns:
            str: ログイン種別
        """
        return self._rx_login_type

    @property
    def javax_view_state(self) -> str:
        """
        View state

        Returns:
            str: View state
        """
        return self._javax_view_state


class Unipa:
    """
    UNIVERSAL PASSPORT 基本ライブラリ
    """

    def __init__(self,
                 base_url: str):
        """
        Unipa クラスを初期化します

        Args:
            base_url: UNIVERSAL PASSPORT のベース URL
                (ログインページ URL にて、 `/up/` より前の URL を指定します。例: https://unipa.itp.kindai.ac.jp/)
        """
        self.session: requests.Session = requests.Session()
        self.session.headers["User-Agent"] = "get-unipa (https://github.com/book000/get-unipa)"
        self.logger = logging.getLogger(__name__)
        self.__base_url: str = base_url
        self.__logged_in: bool = False
        self.__response: Optional[Response] = None

        self.__token: Optional[UnipaToken] = None
        self.__nav_items: List[UnipaNavItem] = []
        self.__info_items: List[UnipaInfoItem] = []

        self.request_url: UnipaRequestUrl = UnipaRequestUrl(base_url)

    def login(self,
              username: str,
              password: str) -> bool:
        """
        UNIVERSAL PASSPORT にログインする

        Args:
            username: ユーザー名
            password: パスワード

        Returns:
            bool: ログインできたか
        """

        self.__response = self.session.get(self.__base_url)
        if self.__response.status_code != 200:
            raise UnipaInternalError("ログインページの取得に失敗しました。")

        soup = BeautifulSoup(self.__response.text, "html5lib")
        login_form = soup.find("form", {"id": "loginForm"})
        if login_form is None:
            # form#loginForm が見つからない場合、ログイントークンなどが取得できないのでログイン不可
            raise UnipaInternalError("ログインフォーム情報の取得に失敗しました。")

        # ログインURL構成 (base_url + actionの値)
        login_url = urljoin(self.__base_url, login_form.get("action"))
        self.logger.debug("ログインフォームのURL: %s", login_url)

        params = {}

        # input と button の値をログイン処理でそのまま流用する
        for input_tag in login_form.find_all("input"):
            input_tag_name = input_tag.get("name")
            input_tag_value = input_tag.get("value")
            if input_tag_name is None or input_tag_value is None:
                continue
            if input_tag_value == "":
                continue

            params[input_tag_name] = input_tag_value

        for button_tag in login_form.find_all("button"):
            button_tag_name = button_tag.get("name")
            button_tag_value = button_tag.get("value")
            if button_tag_name is None:
                continue

            if button_tag_value is None:
                button_tag_value = ""

            params[button_tag_name] = button_tag_value

        self.logger.debug("ログインフォームのデフォルトパラメータ: %s", params)

        params["loginForm:userId"] = username
        params["loginForm:password"] = password

        self.__response = self.session.post(login_url, data=params, headers={
            "Content-Type": "application/x-www-form-urlencoded"
        })

        if self.__response.status_code != 200:
            return False

        soup = BeautifulSoup(self.__response.text, "html5lib")
        error_details = soup.find("span", {"class": "ui-messages-error-detail"})
        if error_details is not None:
            raise UnipaLoginError(error_details.text)

        self.__logged_in = True
        self.update_token_html5lib(soup)

        self.request_url.set("TOP", soup)

        # トップページに移動（アンケートとかがある場合アンケート一覧が出るので）
        soup = self.request("TOP", "headerForm", {
            "rx.sync.source": "headerForm:logo",
            "headerForm:logo": "",
        })

        self.__nav_items = UnipaUtils.get_nav_items(soup)
        self.__info_items = UnipaUtils.get_info_items(soup)

        self.request_url.set("TOP", soup)
        return True

    def request_from_menu(self,
                          menu_item: UnipaNavItem,
                          extra_params: dict[str, str] = None) -> BeautifulSoup:
        """
        メニューからリクエストを送信する

        Args:
            menu_item: メニューアイテム
            extra_params: リクエストに付加するパラメータ (トークンなど以外)

        Returns:
            Response: レスポンス
        """
        if extra_params is None:
            extra_params = {}

        if not self.__logged_in or self.request_url.get("TOP") is None or self.__token is None:
            raise UnipaNotLoggedIn()

        params = {
            "menuForm:mainMenu": "menuForm:mainMenu",
            "rx.sync.source": "menuForm:mainMenu",
            "menuForm:mainMenu_menuid": str(menu_item.menu_id),
        }
        params.update(extra_params)
        return self.request("TOP", "menuForm", params)

    def request_from_info(self,
                          info_item: UnipaInfoItem) -> BeautifulSoup:
        """
        インフォメーションからリクエストを送信する

        Args:
            info_item: インフォメーションアイテム

        Returns:
            Response: レスポンス
        """
        if not self.__logged_in or self.request_url.get("TOP") is None or self.__token is None:
            raise UnipaNotLoggedIn()

        return self.request("TOP", "funcForm", {
            "menuForm:mainMenu": "menuForm:mainMenu",
            "rx.sync.source": info_item.menu_id,
            info_item.menu_id: info_item.menu_id,
        })

    def request(self,
                request_target: str,
                request_type: str,
                extra_params: dict[str, str],
                response_markup: str = "html5lib") -> BeautifulSoup:
        """
        リクエストを送信する

        Args:
            request_target: リクエストターゲット
            request_type: リクエストタイプ (menuForm, funcForm など)
            extra_params: リクエストに付加するパラメータ (トークンなど以外)
            response_markup: レスポンスのマークアップ

        Returns:
            Response: レスポンス
        """
        if not self.__logged_in or self.request_url.get("TOP") is None or self.__token is None:
            raise UnipaNotLoggedIn()

        params = {
            "rx-token": self.__token.rx_token,
            "rx-loginKey": self.__token.rx_login_key,
            "rx-deviceKbn": self.__token.rx_device_kbn,
            "rx-loginType": self.__token.rx_login_type,
            "javax.faces.ViewState": self.__token.javax_view_state,
            request_type: request_type
        }
        params.update(extra_params)

        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
        }
        if response_markup == "lxml":
            headers["Accept"] = "text/html"
        if response_markup == "lxml":
            headers["Accept"] = "application/xml,text/xml"
            headers["Faces-Request"] = "partial/ajax"
            headers["X-Requested-With"] = "XMLHttpRequest"

        url = self.request_url.get(request_target)
        if url is None:
            raise UnipaInternalError("リクエストターゲットが見つかりません: " + request_target)

        self.logger.debug("リクエストURL: %s", url)
        self.logger.debug("リクエストデータ: %s", params)
        self.logger.debug("リクエストヘッダー: %s", headers)
        self.__response = self.session.post(url, data=params, headers=headers)

        if self.__response.status_code != 200:
            self.logger.debug("レスポンス: %s", self.__response.text)
            raise UnipaInternalError("リクエストに失敗しました。(" + str(self.__response.status_code) + ")")

        soup = BeautifulSoup(self.__response.text, response_markup)
        self.logger.debug("soupレスポンス: %s", soup.prettify())

        if response_markup == "html5lib":
            self.update_token_html5lib(soup)

        # xmlのときに更新するかは検討

        return soup

    def is_logged_in(self) -> bool:
        """
        ログインしているかどうかを返します。

        Returns:
            bool: ログインしているかどうか
        """
        return self.__logged_in

    def get_token(self) -> Optional[UnipaToken]:
        """
        トークンを返します。

        Returns:
            Optional[UnipaToken]: トークン
        """
        return self.__token

    def get_nav_items(self) -> List[UnipaNavItem]:
        """
        ナビゲーションアイテムを返します。

        Returns:
            List[UnipaNavItem]: ナビゲーションアイテム
        """
        return self.__nav_items

    def get_info_items(self) -> List[UnipaInfoItem]:
        """
        インフォメーションアイテムを返します。

        Returns:
            List[UnipaInfoItem]: インフォメーションアイテム
        """
        return self.__info_items

    def get_latest_response(self) -> Optional[Response]:
        """
        最後のレスポンスを返します。デバッグのために利用することを想定しています。

        Returns:
            Optional[Response]: レスポンス (ない場合は None)
        """
        return self.__response

    def update_token_html5lib(self,
                              soup: BeautifulSoup) -> None:
        """
        トークンをアップデートします。
        """
        header_form = soup.find("form", {"id": "headerForm"})
        if header_form is None:
            raise UnipaInternalError("トークンを更新するためのフォーム情報の取得に失敗しました。")

        rx_token = header_form.find("input", {"name": "rx-token"}).get("value")
        rx_login_key = header_form.find("input", {"name": "rx-loginKey"}).get("value")
        rx_device_kbn = header_form.find("input", {"name": "rx-deviceKbn"}).get("value")
        rx_login_type = header_form.find("input", {"name": "rx-loginType"}).get("value")
        javax_view_state = header_form.select_one("input[name=\"javax.faces.ViewState\"]").get("value")

        self.__token = UnipaToken(rx_token, rx_login_key, rx_device_kbn, rx_login_type, javax_view_state)



"""
パッケージ: 掲示板
"""
import re
from typing import List, Optional

from unipa import Unipa
from unipa.BulletinBoard.BulletinBoard import UnipaBulletinBoardItem
from unipa.errors import UnipaInternalError, UnipaNotLoggedIn


class UnipaBulletinBoard:
    """
    掲示板
    """

    def __init__(self,
                 unipa: Unipa):
        """
        コンストラクタ
        """
        self.unipa = unipa
        self.logger = unipa.logger

    def get_all(self) -> List[UnipaBulletinBoardItem]:
        """
        掲示リストを取得します。

        Returns:
            List[UnipaBulletinBoardItem]: 掲示リスト
        """

        if not self.unipa.is_logged_in():
            raise UnipaNotLoggedIn()

        nav_item = next(filter(lambda x: x.name == "掲示板", self.unipa.get_nav_items()))
        menu_id = nav_item.menu_id

        self.logger.debug(f"BulletinBoard/menu_id: {menu_id}")

        soup = self.unipa.request_from_menu(nav_item, {
            "javax.faces.partial.execute": "@all"
        })
        self.unipa.request_url.set("BULLETBOARD", soup)

        if soup.find("div", {"class": "ui-tabs-panels"}) is None:
            raise UnipaInternalError("掲示板パネルが見つかりませんでした。")

        panels = soup.select("div.ui-tabs-panels > div[role=\"tabpanel\"]")

        if len(panels) < 2:
            raise UnipaInternalError("掲示板パネルのうち、全表示タブが見つかりませんでした。")

        panel = panels[1]

        # このへんの処理あまりにも無理やりなのですぐ壊れるかも

        items = []
        # div.ui-scrollpanel > div.alignRight
        for item in panel.select("div.ui-scrollpanel > div.alignRight"):
            a_tag = item.find("a")
            item_id = a_tag.get("id")
            title = a_tag.text
            onclick = a_tag.get("onclick")
            [target_s, target_p] = self.get_target_sp(onclick)  # idを拾ってもいい
            if target_s is None or target_p is None:
                continue

            is_attention = item.select_one("i.iconColorAttention") is not None

            buttons = panel.select("span.inlineBlock > div[type=\"button\"]")
            if len(buttons) < 2:
                raise UnipaInternalError("掲示板パネルのうち、フラグ・未/既読ボタンが見つかりませんでした。")

            flag_input = buttons[0].select_one("input[type=\"checkbox\"]")
            flag_id = flag_input.get("id")
            is_flag = flag_input.get("checked") is None

            unread_input = buttons[1].select_one("input[type=\"checkbox\"]")
            unread_id = unread_input.get("id")
            is_unread = unread_input.get("checked") is not None

            items.append(UnipaBulletinBoardItem(
                item_id,
                title,
                target_s,
                target_p,
                flag_id,
                unread_id,
                is_attention,
                is_flag,
                is_unread
            ))

        return items

    @staticmethod
    def get_target_sp(onclick: str) -> List[Optional[str]]:
        """
        onclick 属性から、s, pを取得します。

        Args:
            onclick: onclick 属性の値

        Returns:
            [s, p]: s, p 値
        """
        pattern = re.compile(r"PrimeFaces\.ab\({s:\"(.+?)\",p:\"(.+?)\"}\);")
        match = pattern.search(onclick)
        if match is None:
            return [None, None]

        return [match.group(1), match.group(2)]



"""
掲示板
"""
from unipa import Unipa, UnipaUtils
from unipa.BulletinBoard.BulletinBoardItemDetails import UnipaBulletinBoardItemDetails
from unipa.BulletinBoard.PublicationPeriod import UnipaPublicationPeriod


class UnipaBulletinBoardItem:
    """
    掲示板の掲示アイテム
    """

    def __init__(self,
                 item_id: str,
                 title: str,
                 target_s: str,
                 target_p: str,
                 flag_id: str,
                 unread_id: str,
                 is_attention: bool,
                 is_flag: bool,
                 is_unread: bool):
        """
        掲示板の掲示アイテム コンストラクタ

        Args:
            item_id: 掲示 ID (恒久的なIDではないことに注意)
            title: 掲示タイトル
            target_s: 掲示 s 値 (リクエスト用)
            target_p: 掲示 p 値 (リクエスト用)
            flag_id: フラグ ID
            unread_id: 未読 ID
            is_attention: 注目フラグ (投稿者によって設定)
            is_flag: フラグ (ユーザーが設定可能)
            is_unread: 未読か (ユーザーが設定可能)
        """
        self._item_id = item_id
        self._title = title
        self._target_s = target_s
        self._target_p = target_p
        self._flag_id = flag_id
        self._unread_id = unread_id
        self._is_attention = is_attention
        self._is_flag = is_flag
        self._is_unread = is_unread

    @property
    def item_id(self) -> str:
        """
        掲示 ID

        Returns:
            str: 掲示 ID

        Notes:
            この ID は恒久的な ID ではなく、掲示表示順番のみで ID が振られているようです。<br>
            <b>掲示内容の変化検知など、掲示が合致するかどうかのチェックには用いないでください</b>
        """
        return self._item_id

    @property
    def title(self) -> str:
        """
        掲示タイトル

        Returns:
            str: 掲示タイトル
        """
        return self._title

    @property
    def target_s(self) -> str:
        """
        掲示 s 値 (リクエスト用)

        Returns:
            str: 掲示 s 値 (リクエスト用)
        """
        return self._target_s

    @property
    def target_p(self) -> str:
        """
        掲示 p 値 (リクエスト用)

        Returns:
            str: 掲示 t 値 (リクエスト用)
        """
        return self._target_p

    @property
    def flag_id(self) -> str:
        """
        フラグ ID

        Returns:
            str: フラグ ID
        """
        return self._flag_id

    @property
    def unread_id(self) -> str:
        """
        未読 ID

        Returns:
            str: 未読 ID
        """
        return self._unread_id

    @property
    def is_attention(self) -> bool:
        """
        注目フラグ (投稿者によって設定)

        Returns:
            bool: 注目フラグ (投稿者によって設定)
        """
        return self._is_attention

    @property
    def is_flag(self) -> bool:
        """
        フラグ (ユーザーが設定可能)

        Returns:
            bool: フラグ (ユーザーが設定可能)
        """
        return self._is_flag

    @property
    def is_unread(self) -> bool:
        """
        未読か (ユーザーが設定可能)

        Returns:
            bool: 未読か (ユーザーが設定可能)
        """
        return self._is_unread

    def get_details(self,
                    unipa: Unipa) -> UnipaBulletinBoardItemDetails:
        """
        掲示アイテムの詳細を取得します。

        Returns:
            UnipaBulletinBoardItemDetails: 掲示アイテムの詳細
        """
        soup = unipa.request("BULLETBOARD", "funcForm", {
            "javax.faces.source": self.target_s,
            "javax.faces.partial.execute": self.target_s,
            "funcForm:tabArea_activeIndex": "1",
            self.target_s: self.target_p
        }, "html5lib")

        outputpanel = soup.select_one("div.ui-outputpanel")
        if outputpanel is None:
            raise RuntimeError("掲示板の詳細パネルが見つかりません。")

        table = outputpanel.select_one("table.singleTable")
        data = {}
        for tr in table.find_all("tr"):
            tds = tr.find_all("td")
            if len(tds) != 2:
                continue

            row_key = tds[0]
            row_value = tds[1]

            # print(row_key.text, row_value.text)
            data[row_key.text.strip()] = row_value

        raw_publication_period = data["掲示期間"]
        raw_start_date: str = raw_publication_period.find_all("span")[0].text.strip()
        raw_end_date = raw_publication_period.find_all("span")[2].text.strip()

        start_date = UnipaUtils.process_datetime(raw_start_date)
        end_date = UnipaUtils.process_datetime(raw_end_date)

        publication_period = UnipaPublicationPeriod(start_date, end_date)

        return UnipaBulletinBoardItemDetails(
            item_id=self.item_id,
            title=data["件名"].text.strip(),
            author=data["差出人"].text.strip(),
            category=data["カテゴリ"].text.strip(),
            content_html=str(data["本文"]),
            publication_period=publication_period,
        )

    def mark_read(self,
                  unipa: Unipa) -> None:
        """
        既読にする
        """
        raise NotImplementedError("未実装です。")  # TODO

    def mark_unread(self,
                    unipa: Unipa) -> None:
        """
        未読にする
        """
        raise NotImplementedError("未実装です。")  # TODO

    def mark_flag(self,
                  unipa: Unipa) -> None:
        """
        フラグをつける
        """
        raise NotImplementedError("未実装です。")  # TODO

    def mark_unflag(self,
                    unipa: Unipa) -> None:
        """
        フラグをはずす
        """
        raise NotImplementedError("未実装です。")  # TODO

    def __str__(self) -> str:
        return f"BulletinBoardItem(title={self.title}, " \
               f"target_s={self.target_s}, " \
               f"target_p={self.target_p}, " \
               f"is_attention={self.is_attention}, " \
               f"is_flag={self.is_flag}, " \
               f"is_unread={self.is_unread})"



"""
掲示板の掲示アイテム 詳細情報
"""
from unipa.BulletinBoard.PublicationPeriod import UnipaPublicationPeriod


class UnipaBulletinBoardItemDetails:
    """
    掲示板の掲示アイテム 詳細情報

    Notes:
        get-unipa ではコンテンツ内容のプレーンテキスト化はしていません。`content_html` プロパティで HTML をそのまま返すため、Markdown などへの変換は利用側で実施してください。
    """

    def __init__(self,
                 item_id: str,
                 title: str,
                 author: str,
                 category: str,
                 content_html: str,
                 publication_period: UnipaPublicationPeriod):
        """
        コンストラクタ

        Args:
            item_id: 掲示 ID (恒久的なIDではないことに注意)
            title: 掲示タイトル
            author: 差出人
            category: カテゴリ
            content_html: コンテンツ HTML
            publication_period: 公開期間
        """
        self._item_id = item_id
        self._title = title
        self._author = author
        self._category = category
        self._content_html = content_html
        self._publication_period = publication_period

    @property
    def item_id(self) -> str:
        """
        掲示 ID

        Returns:
            str: 掲示 ID

        Notes:
            この ID は恒久的な ID ではなく、掲示表示順番のみで ID が振られているようです。<br>
            <b>掲示内容の変化検知など、掲示が合致するかどうかのチェックには用いないでください</b>
        """
        return self._item_id

    @property
    def title(self) -> str:
        """
        掲示タイトル

        Returns:
            str: 掲示タイトル
        """
        return self._title

    @property
    def author(self) -> str:
        """
        差出人

        Returns:
            str: 差出人
        """
        return self._author

    @property
    def category(self) -> str:
        """
        カテゴリ

        Returns:
            str: カテゴリ
        """
        return self._category

    @property
    def content_html(self) -> str:
        """
        コンテンツ HTML

        Returns:
            str: コンテンツ HTML
        """
        return self._content_html

    @property
    def publication_period(self) -> UnipaPublicationPeriod:
        """
        公開期間

        Returns:
            PublicationPeriod: 公開期間
        """
        return self._publication_period

    def __str__(self) -> str:
        return f"UnipaBulletinBoardItemDetails(title={self._title}, " \
               f"author={self._author}, " \
               f"category={self._category}, " \
               f"content_html={self._content_html}, " \
               f"publication_period={self._publication_period})"



"""
get-unipa エラーException群
"""


class UnipaInternalError(Exception):
    """
    処理に失敗した
    """


class UnipaLoginError(Exception):
    """
    ログインに失敗した
    """


class UnipaNotLoggedIn(Exception):
    """
    UNIPA にログインしている必要があるがしていない
    """



"""
公開期間情報 (掲示板掲示用)
"""
import datetime
from typing import Optional


class UnipaPublicationPeriod:
    """
    公開期間情報 (掲示板掲示用)
    """

    def __init__(self,
                 start_date: Optional[datetime.datetime],
                 end_date: Optional[datetime.datetime]):
        """
        公開期間情報 (掲示板掲示用) を生成する。

        Args:
            start_date: 公開開始日
            end_date: 公開終了日
        """
        self._start_date = start_date
        self._end_date = end_date

    @property
    def start_date(self) -> Optional[datetime.datetime]:
        """
        公開開始日

        Returns:
            Optional[datetime.datetime]: 公開開始日
        """
        return self._start_date

    @property
    def end_date(self) -> Optional[datetime.datetime]:
        """
        公開終了日

        Returns:
            Optional[datetime.datetime]: 公開終了日
        """
        return self._end_date



"""
クラス 基本情報

クラスプロファイルより取得
"""
from typing import Optional


class UnipaClassLectureAt:
    """
    クラス 曜日・時限情報
    """

    def __init__(self,
                 day_of_week: str,
                 period_of_time: Optional[int]):
        """

        Args:
            day_of_week: 実施曜日 (月曜日 etc or 集中講義 or 実習)
            period_of_time: 実施時限 (数値)
        """
        self._day_of_week = day_of_week
        self._period_of_time = period_of_time

    @property
    def day_of_week(self) -> str:
        """
        実施曜日 (月曜日 etc or 集中講義 or 実習)

        Returns:
            str: 実施曜日 (月曜日 etc or 集中講義 or 実習)
        """
        return self._day_of_week

    @property
    def period_of_time(self) -> Optional[int]:
        """
        実施時限 (数値)

        Returns:
            Optional[int]: 実施時限 (数値)
        """
        return self._period_of_time

    def __str__(self) -> str:
        return f"UnipaClassLectureAt(day_of_week={self._day_of_week}, period_of_time={self._period_of_time})"


class UnipaClass:
    """
    クラス 基本情報
    """

    def __init__(self,
                 class_internal_id: str,
                 name: str,
                 class_id: Optional[str],
                 lecture_at: UnipaClassLectureAt):
        """
        コンストラクタ

        Args:
            class_internal_id: クラス内部ID
            name: クラス名
            class_id: クラスID (表示されている値)
            lecture_at: 実施曜日・時限
        """
        self._class_internal_id = class_internal_id
        self._name = name
        self._class_id = class_id
        self._lecture_at = lecture_at

    @property
    def class_internal_id(self) -> str:
        """
        クラス内部ID

        Returns:
            str: クラス内部ID
        """
        return self._class_internal_id

    @property
    def name(self) -> str:
        """
        クラス名

        Returns:
            str: クラス名
        """
        return self._name

    @property
    def class_id(self) -> Optional[str]:
        """
        クラスID (表示されている値)

        Returns:
            Optional[str]: クラスID (表示されている値)
        """
        return self._class_id

    @property
    def lecture_at(self) -> UnipaClassLectureAt:
        """
        実施曜日・時限

        Returns:
            UnipaClassLectureAt: 実施曜日・時限
        """
        return self._lecture_at

    def __str__(self) -> str:
        return f"UnipaClass(class_internal_id={self._class_internal_id}, " \
               f"name={self._name}, " \
               f"class_id={self._class_id}, " \
               f"lecture_at={self._lecture_at})"



"""
テストデータ作成
"""
import datetime
import logging
import os
from typing import List

from unipa import Unipa
from unipa.BulletinBoard import UnipaBulletinBoard, UnipaBulletinBoardItem
from unipa.models.tests import BulletinBoardGetAllModel, BulletinBoardGetDetailsModel, ConfigJsonModel, \
    PublicationPeriodModel, TestsJsonModel


class GenerateTestData:
    """
    テストデータ作成
    """

    def __init__(self) -> None:
        self.logger = logging.getLogger(__name__)

        if not os.path.exists("config.json"):
            raise Exception("テストに必要なファイル config.json が見つかりません")

        with open("config.json", encoding="utf-8") as f:
            self.config: ConfigJsonModel = ConfigJsonModel.from_json(f.read())  # type: ignore

        self.unipa = Unipa(self.config.base_url)
        self.unipa.login(self.config.username, self.config.password)

        self.logger.info("テストデータを作成します")
        with open("tests.json", "w", encoding="utf-8") as f:
            f.write(self.generate().to_json(ensure_ascii=False, indent=4))  # type: ignore

    def generate(self) -> TestsJsonModel:
        """

        Returns:

        """
        bulletinboard_items = UnipaBulletinBoard(self.unipa).get_all()
        bulletinboard_get_all = self.get_bulletinboard_get_all(bulletinboard_items)
        bulletinboard_get_details = self.get_bulletinboard_get_details(bulletinboard_items)

        return TestsJsonModel(
            bulletinboard_get_all=bulletinboard_get_all,
            bulletinboard_get_details=bulletinboard_get_details,
        )

    @classmethod
    def get_bulletinboard_get_all(cls,
                                  bulletinboard_items: List[UnipaBulletinBoardItem]) -> List[BulletinBoardGetAllModel]:
        """
        掲示板 掲示リスト

        Args:
            bulletinboard_items (List[UnipaBulletinBoardItem]): UnipaBulletinBoard.get_all
        """
        ret = []

        for item in bulletinboard_items:
            ret.append(BulletinBoardGetAllModel(
                item_id=item.item_id,
                title=item.title,
                target_s=item.target_s,
                target_p=item.target_p,
                flag_id=item.flag_id,
                unread_id=item.unread_id,
                is_attention=item.is_attention,
                is_flag=item.is_flag,
                is_unread=item.is_unread,
            ))

        return ret

    def get_bulletinboard_get_details(self,
                                      bulletinboard_items: List[UnipaBulletinBoardItem]) \
            -> List[BulletinBoardGetDetailsModel]:
        """
        掲示板 掲示詳細
        """
        ret = []

        for item in bulletinboard_items:
            details = item.get_details(self.unipa)
            ret.append(BulletinBoardGetDetailsModel(
                item_id=item.item_id,
                title=details.title,
                author=details.author,
                category=details.category,
                content_html=details.content_html,
                publication_period=PublicationPeriodModel(
                    start_date=self.datetime2str(details.publication_period.start_date),
                    end_date=self.datetime2str(details.publication_period.end_date),
                ),
            ))

        return ret

    @classmethod
    def datetime2str(cls,
                     dt: datetime.datetime) -> str:
        """
        datetime を str に変換
        """
        day_of_week = '月火水木金土日'
        return dt.strftime("%Y/%m/%d(DAYOFWEEK) %H:%M").replace("DAYOFWEEK", day_of_week[dt.weekday()])


def main() -> None:
    """
    テストデータ作成 メイン関数
    """
    GenerateTestData()


if __name__ == '__main__':
    main()



"""
ユーティリティ
"""
import datetime
import re
from typing import Dict, List, Optional
from urllib.parse import urljoin

from bs4 import BeautifulSoup

from unipa import UnipaInternalError


class Menu:
    """
    ナビゲーションメニュー
    """

    def __init__(self,
                 name: str,
                 sub_name: Optional[str]):
        """
        ナビゲーションメニュー コンストラクタ

        Args:
            name: メニュー名
            sub_name: サブメニュー名
        """
        self._name = name
        self._sub_name = sub_name

    @property
    def name(self) -> str:
        """
        メニュー名

        Returns:
            str: メニュー名
        """
        return self._name

    @property
    def sub_name(self) -> Optional[str]:
        """
        サブメニュー名

        Returns:
            Optional[str]: サブメニュー名
        """
        return self._sub_name

    def __str__(self) -> str:
        return f"Menu(_name={self.name}, sub__name={self.sub_name})"


class UnipaNavItem:
    """
    ナビゲーションアイテム
    """

    def __init__(self,
                 menu: Menu,
                 name: str,
                 menu_id: Optional[str]):
        """
        ナビゲーションアイテム コンストラクタ

        Args:
            menu: メニュー
            name: アイテム名
            menu_id: メニュー ID
        """
        self._menu = menu
        self._name = name
        self._menu_id = menu_id

    @property
    def menu(self) -> Menu:
        """
        メニュー

        Returns:
            Menu: メニュー
        """
        return self._menu

    @property
    def name(self) -> str:
        """
        アイテム名

        Returns:
            str: アイテム名
        """
        return self._name

    @property
    def menu_id(self) -> Optional[str]:
        """
        メニュー ID

        Returns:
            Optional[str]: メニュー ID
        """
        return self._menu_id

    def __str__(self) -> str:
        return f"UnipaNavItem(_menu={self._menu}, _name={self._name}, __menu_id={self._menu_id})"


class UnipaInfoItem:
    """
    トップインフォメーションアイテム
    """

    def __init__(self,
                 name: str,
                 item_id: str):
        """
        トップインフォメーションアイテム コンストラクタ

        Args:
            name: アイテム名
            item_id: アイテム ID
        """
        self._name = name
        self._item_id = item_id

    @property
    def name(self) -> str:
        """
        アイテム名

        Returns:
            str: アイテム名
        """
        return self._name

    @property
    def menu_id(self) -> str:
        """
        アイテム ID

        Returns:
            str: アイテム ID
        """
        return self._item_id

    def __str__(self) -> str:
        return f"UnipaInfoItem(_name={self._name}, _item_id={self._item_id})"


class UnipaRequestUrl:
    """
    リクエスト URL の列挙

    フォームのaction属性を取得し更新する。システムによって違うかもしれないので
    """
    __urls: Dict[str, Optional[str]] = {
        "TOP": None,
        "BULLETBOARD": None,
        "SITEMAP": None,
    }

    def __init__(self,
                 base_url: str):
        """
        UnipaRequestUrl コンストラクタ

        Args:
            base_url: ベース URL
        """
        self.__base_url = base_url

    def get(self,
            name: str) -> Optional[str]:
        """
        UnipaRequestUrl の値 (URL) を取得する

        Args:
            name: UnipaRequestUrl の名前

        Returns:
            Optional[str]: UnipaRequestUrl の値 (URL)
        """
        if name not in self.__urls:
            raise UnipaInternalError(f"{name} is undefined")

        if self.__urls[name] is None:
            return None

        return urljoin(self.__base_url, self.__urls[name])

    def set(self,
            name: str,
            soup: BeautifulSoup) -> None:
        """
        BeautifulSoup から UnipaRequestUrl の値 (URL) を設定する

        Args:
            name: UnipaRequestUrl の名前
            soup: BeautifulSoup
        """
        if name not in self.__urls:
            raise UnipaInternalError(f"{name} is undefined")

        header_form = soup.find("form", {"id": "headerForm"})
        self.__urls[name] = header_form.get("action")


class UnipaUtils:
    """
    ユーティリティ
    """

    def __init__(self) -> None:
        pass

    @classmethod
    def get_nav_items(cls,
                      soup: BeautifulSoup) -> List[UnipaNavItem]:
        """
        ナビゲーションアイテムを取得する

        Args:
            soup: BeautifulSoup

        Returns:
            List[UnipaNavItem]: ナビゲーションアイテム
        """
        menu_form = soup.find("div", {"id": "menuForm:mainMenu"})
        if menu_form is None:
            raise UnipaInternalError("メニューが見つかりません")

        items = []
        menu_items = soup.select("div[id=\"menuForm:mainMenu\"] > .ui-menu-list > .ui-menuitem")
        for menu_item in menu_items:
            menu_name = menu_item.select_one("a.ui-submenu-link").find("span", {"class": "ui-menuitem-text"})

            submenus = menu_item.select_one("ul.ui-menu-child").find_all("td")

            for submenu in submenus:
                submenu_name = submenu.select_one("li.ui-widget-header").find("h3")
                submenu_items = submenu.select("li.ui-menuitem")
                for submenu_item in submenu_items:
                    pfconfirmcommand = submenu_item.select_one("a.ui-menuitem-link").get("data-pfconfirmcommand")
                    if pfconfirmcommand is None:
                        continue
                    items.append(UnipaNavItem(
                        menu=Menu(
                            name=menu_name.text,
                            sub_name=submenu_name.text
                        ),
                        name=submenu_item.find("span", {"class": "ui-menuitem-text"}).text,
                        menu_id=cls.get_menuid(pfconfirmcommand)
                    ))

        return items

    @classmethod
    def get_info_items(cls,
                       soup: BeautifulSoup) -> List[UnipaInfoItem]:
        """
        インフォメーションアイテムを取得する

        Args:
            soup: BeautifulSoup

        Returns:
            List[UnipaInfoItem]: インフォメーションアイテム
        """
        info_detail = soup.select_one("#portalCont .infoDetail")  #
        if info_detail is None:
            raise UnipaInternalError("ポータルコンテナ/インフォメーションが見つかりません")

        info_items = info_detail.select("div.ui-panel-content")

        with open("hoge.html", "w") as f:
            f.write(str(info_items))

        items = []
        for info_item in info_items:
            title = info_item.find("span", {"class": "span"}).text
            item_id = info_item.select_one("a.ui-commandlink").get("id")

            print(title, item_id)

            items.append(UnipaInfoItem(
                name=title,
                item_id=item_id
            ))

        return items

    @staticmethod
    def get_menuid(pfconfirmcommand: str) -> Optional[str]:
        """
        メニュー ID を取得する

        Args:
            pfconfirmcommand: data-pfconfirmcommand

        Returns:
            Optional[str]: メニュー ID
        """
        pattern = re.compile(r"'menuForm:mainMenu_menuid':'(.+?)'")
        match = pattern.search(pfconfirmcommand)
        if match is None:
            return None

        return match.group(1)

    @staticmethod
    def process_datetime(datetime_str: Optional[str]) -> Optional[datetime.datetime]:
        """
        UNIPA の日時テキスト(YYYY/MM/DD HH:MM)から datetime.datetime に変換する

        Args:
            datetime_str: UNIPA の日時テキスト

        Returns:
            Optional[datetime.datetime]: 変換後の datetime.datetime
        """
        if datetime_str is None or datetime_str == "":
            return None
        datetime_str = re.sub(r"\(.+?\)", "", datetime_str)
        datetime_format = "%Y/%m/%d %H:%M %z"
        jst = datetime.timezone(datetime.timedelta(hours=+9), 'JST')
        return datetime.datetime.strptime(datetime_str + " +0900", datetime_format).astimezone(jst)



import setuptools

setuptools.setup(
    name='get-unipa',
    version=open("get-unipa.version").read().strip(),
    packages=setuptools.find_packages(),
    install_requires=["beautifulsoup4", "requests", "html5lib"],
    url='https://github.com/book000/get-unipa',
    license='MIT',
    author='Tomachi',
    author_email='tomachi@tomacheese.com',
    maintainer='Tomachi',
    maintainer_email='tomachi@tomacheese.com',
    description='Library for get various information about UNIVERSAL PASSPORT.',
    long_description=open('README.md', encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: MIT License',
    ],
)



"""
ユニットテスト
"""
import logging
import os
import sys
from unittest import TestCase

from unipa import Unipa, UnipaLoginError, UnipaUtils
from unipa.BulletinBoard import UnipaBulletinBoard
from unipa.models.tests import ConfigJsonModel, TestsJsonModel


class TestUnipa(TestCase):
    """
    ユニットテスト
    """

    # noinspection PyPep8Naming
    def setUp(self) -> None:
        """
        テストセットアップ
        """
        self.logger = logging.getLogger(__name__)
        if not self.logger.hasHandlers():
            sh = logging.StreamHandler(sys.stdout)
            self.logger.setLevel(logging.DEBUG)
            self.logger.addHandler(sh)

        if not os.path.exists("config.json"):
            self.fail("テストに必要なファイル config.json が見つかりません")

        with open("config.json", encoding="utf-8") as f:
            self.config: ConfigJsonModel = ConfigJsonModel.from_json(f.read())  # type: ignore

        self.logger.info("config.json の読み込みに成功しました")

        if not os.path.exists("tests.json"):
            self.fail("テストに必要なファイル tests.json が見つかりません")

        with open("tests.json", encoding="utf-8") as f:
            self.tests: TestsJsonModel = TestsJsonModel.from_json(f.read())  # type: ignore

        self.logger.info("tests.json の読み込みに成功しました")

        # ------------------------------------------------------------ #

        self.base_url = self.config.base_url
        self.unipa = Unipa(self.base_url)

        self.assertRaises(UnipaLoginError, self.unipa.login, "XXXXXXXX", "XXXXXXXX")

        username = self.config.username
        password = self.config.password

        self.assertTrue(self.unipa.login(username, password), "ログインに失敗しました")

        self.bulletinboard_items = UnipaBulletinBoard(self.unipa).get_all()

    def test_bulletinboard_get_all(self) -> None:
        """
        掲示リストの取得テスト
        """

        for test in self.tests.bulletinboard_get_all:
            self.logger.info("掲示リスト取得テスト: タイトル「%s」", test.title)

            target = None
            target_match = 0
            for match in self.bulletinboard_items:
                results = [
                    match.title == test.title,
                    match.flag_id == test.flag_id,
                    match.unread_id == test.unread_id,
                    match.is_attention == test.is_attention,
                    match.is_flag == test.is_flag,
                    match.is_unread == test.is_unread,
                ]
                if len(list(filter(lambda result: result is True, results))) > target_match:
                    target = match
                    target_match = len(list(filter(lambda result: result is True, results)))

            if target is None:
                self.fail("掲示リストに該当する掲示が見つかりませんでした。")

            self.assertEqual(target.title, test.title, "タイトルが一致しません")
            self.assertEqual(target.flag_id, test.flag_id, "flag_idが一致しません")
            self.assertEqual(target.unread_id, test.unread_id, "unread_idが一致しません")
            self.assertEqual(target.is_attention, test.is_attention, "is_attentionが一致しません")
            self.assertEqual(target.is_flag, test.is_flag, "is_flagが一致しません")
            self.assertEqual(target.is_unread, test.is_unread, "is_unreadが一致しません")

    def test_bulletinboard_get_details(self) -> None:
        """
        掲示板 掲示詳細の取得テスト
        """
        for test in self.tests.bulletinboard_get_details:
            self.logger.info("掲示板 掲示詳細の取得テスト: 掲示タイトル「%s」", test.title)

            target = None
            target_match = 0
            for match in self.bulletinboard_items:
                details = match.get_details(self.unipa)
                results = [
                    details.title == test.title,
                    details.author == test.author,
                    details.category == test.category,
                    details.content_html == test.content_html,
                    details.publication_period.start_date == UnipaUtils.process_datetime(
                        test.publication_period.start_date),
                    details.publication_period.end_date == UnipaUtils.process_datetime(
                        test.publication_period.end_date),
                ]
                if len(list(filter(lambda result: result is True, results))) > target_match:
                    target = details
                    target_match = len(list(filter(lambda result: result is True, results)))

            if target is None:
                self.fail("掲示リストに該当する掲示が見つかりませんでした。")

            self.assertEqual(target.item_id, test.item_id, "IDが一致しません")
            self.assertEqual(target.title, test.title, "タイトルが一致しません")
            self.assertEqual(target.author, test.author, "authorが一致しません")
            self.assertEqual(target.category, test.category, "categoryが一致しません")
            self.assertEqual(target.content_html, test.content_html, "content_htmlが一致しません")
            self.assertEqual(target.publication_period.start_date,
                             UnipaUtils.process_datetime(test.publication_period.start_date),
                             "publication_period.start_dateが一致しません")
            self.assertEqual(target.publication_period.end_date,
                             UnipaUtils.process_datetime(test.publication_period.end_date),
                             "publication_period.end_dateが一致しません")
